
import com.sun.media.ui.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

class PlayerGui
	extends JFrame
	implements ActionListener, AdjustmentListener, TimeObserver

{
	private static final long serialVersionUID = 1L;
	
	FileDialog fileDialog = new FileDialog(this, "Open File", FileDialog.LOAD);
	PlayerFacade player = null;
	URL url = null;
	JLabel mediaLabel;
	JLabel totalValueLabel;
	JLabel elapsedTimeValue;
	JScrollBar timeline;

	public PlayerGui(PlayerFacade player)
	{
		super("Java Media Framework");

		this.player = player;

		setLayout(new BorderLayout());
		setJMenuBar(createMenuBar());

		JPanel panel = new JPanel(new BorderLayout());

		mediaLabel = new JLabel("Media...");
		panel.add("North", mediaLabel);
		
		JPanel panel2 = new JPanel(new BorderLayout());
		JLabel label = new JLabel("Total:");
		panel2.add("West", label);
		
		totalValueLabel = new JLabel();
		panel2.add("East", totalValueLabel);
		panel.add("Center", panel2);
		
		panel2 = new JPanel(new BorderLayout());
		label = new JLabel("Elapsed:");
		panel2.add("West", label);
		
		elapsedTimeValue = new JLabel();
		panel2.add("East", elapsedTimeValue);
		panel.add("South", panel2);
		add("North", panel);

		panel = new JPanel(new BorderLayout());
		label = new JLabel("Avance:");
		panel.add("West", label);

		timeline = new JScrollBar(
				JScrollBar.HORIZONTAL, 0, 5, 0, 100);
		
		timeline.setSize(100, 20);
		timeline.setName("Tiempo");
		timeline.addAdjustmentListener(this);
		
		panel.add("Center", timeline);
		add("Center", panel);

		panel = new JPanel(new BorderLayout());
		panel.add("North", createButtons());
		
		label = new JLabel("Volume:");
		panel.add("West", label);

		JScrollBar volumeSb = new JScrollBar(
				JScrollBar.HORIZONTAL, 95, 5, 0, 100);
		
		volumeSb.setSize(100, 20);
		volumeSb.setName("Volume");
		volumeSb.addAdjustmentListener(this);
		
		panel.add("Center", volumeSb);          
		add("South", panel);

		pack();
		
		setVisible(true);
		setDefaultCloseOperation(EXIT_ON_CLOSE);

		elapsedTimeValue.setText("0:00");
		totalValueLabel.setText("0:00");
	}

	/**
	 * Sets up a menu bar for the media player.
	 * @return Already built menu bar
	 */
	private JMenuBar createMenuBar()
	{
		JMenuItem item;
		JMenuBar menuBar = new JMenuBar();

		// File Menu
		// =========================

		JMenu fileMenu = new JMenu("File");
		
		item = new JMenuItem("Open");
		item.addActionListener(this);
		fileMenu.add(item);
		
		item = new JMenuItem("Exit");
		item.addActionListener(this);
		fileMenu.add(item);
		
		menuBar.add(fileMenu);

		// Options Menu
		// =========================

		JMenu optionsMenu = new JMenu("Options");
		
		JCheckBoxMenuItem checkBox = new JCheckBoxMenuItem("Repeat");
		checkBox.setState(true);
		checkBox.addActionListener(this);
		optionsMenu.add(checkBox);
		
		checkBox = new JCheckBoxMenuItem("Mute");
		checkBox.setState(false);
		checkBox.addActionListener(this);
		optionsMenu.add(checkBox);
		
		// Speed Menu
		// =========================

		JMenu speedMenu = new JMenu("Speed");
		
		item = new JMenuItem("Slow");
		item.addActionListener(this);
		speedMenu.add(item);
		
		item = new JMenuItem("Normal");
		item.addActionListener(this);
		speedMenu.add(item);
		
		item = new JMenuItem("Fast");
		item.addActionListener(this);
		speedMenu.add(item);
		
		optionsMenu.add(speedMenu);
		menuBar.add(optionsMenu);

		return menuBar;
	}			

	/**
	 * Sets up media player's control buttons
	 * @return Button panel
	 */
	private JPanel createButtons()
	{
		JPanel panel = new JPanel();

		JButton rewindBtn = new JButton("Rewind");
		rewindBtn.addActionListener(this);
		panel.add(rewindBtn);
		
		JButton playBtn = new JButton("Play");
		playBtn.addActionListener(this);
		panel.add(playBtn);
		
		JButton pauseBtn = new JButton("Pause");
		pauseBtn.addActionListener(this);
		panel.add(pauseBtn);
		
		JButton stopBtn = new JButton("Stop");
		stopBtn.addActionListener(this);
		panel.add(stopBtn);
		
		JButton ffBtn = new JButton("Forward");
		ffBtn.addActionListener(this);
		panel.add(ffBtn);
		
		return panel;
	}
	
	// ========================================================================
	//                          ACTION HANDLERS
	// ========================================================================

	private void onOpen()
	{
		fileDialog.setVisible(true);
		
		if (fileDialog.getFile() != null)
		{
			String filename = new String("file:" +
					fileDialog.getDirectory() + fileDialog.getFile());
			
			mediaLabel.setText(fileDialog.getFile());
			
			try
			{
				url = new URL(filename);
				player.setUrl(url);                        
			}
			catch(MalformedURLException e)
			{
				new MessageBox("ERROR", e.toString());
			}
		}
	}
	
	public void actionPerformed(ActionEvent ae)
	{
		String command = ae.getActionCommand();

		if (command.equals("Open"))
			onOpen();

		else if (command.equals("Exit"))
		{
			dispose();
			System.exit(0);
		}
		else if (command.equals("Repeat"))
		{
			JCheckBoxMenuItem checkBox = (JCheckBoxMenuItem)ae.getSource();
			player.setRepeatMode(checkBox.getState());
		}
		else if (command.equals("Mute"))
		{
			JCheckBoxMenuItem checkBox = (JCheckBoxMenuItem)ae.getSource();
			player.setMute(checkBox.getState());
		}
		else if (command.equals("Fast"))
			player.setRate((float)2);

		else if (command.equals("Normal"))
			player.setRate((float)1);

		else if (command.equals("Slow"))
			player.setRate((float)0.5);

		else if (command.equals("Rewind"))
			player.rewind();

		else if (command.equals("Play"))
			player.start();

		else if (command.equals("Pause"))
			player.pause();

		else if (command.equals("Stop")) 
			player.stop();

		else if (command.equals("Forward"))
			player.forward();
	}

	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		JScrollBar scrollBar = (JScrollBar)e.getAdjustable();
		
		if(scrollBar.getName().equals("Tiempo"))
			player.setPosition(scrollBar.getValue());
		else if(scrollBar.getName().equals("Volume"))
			player.setVolume(scrollBar.getValue());            
	}

	/**
	 * Interface against media player to get updates on changes current
	 * media position.
	 */
	public void updateTime(int elapsedSecs)
	{
		if(player != null)
		{
			int position = (int)player.getMediaTime().getSeconds();
			int m = position / 60;
			int s = position % 60;
			
			if(s < 10)
			{
				elapsedTimeValue.setText(Integer.toString(m) +
						":0" + Integer.toString(s));
			}
			else
			{
				elapsedTimeValue.setText(Integer.toString(m) +
						":" + Integer.toString(s));
			}
			timeline.removeAdjustmentListener(this);
			timeline.setValue(position);
			timeline.addAdjustmentListener(this);
		}
	}

	/**
	 * Interface against media player to get updates on changes in media
	 * duration.
	 */
	public void updateTotalTime(int totalSecs)
	{
		int m = totalSecs / 60;
		int s = totalSecs % 60;
		
		if(s < 10)
			totalValueLabel.setText(Integer.toString(m) +
					":0" + Integer.toString(s) );
		else
			totalValueLabel.setText(Integer.toString(m) +
					":" + Integer.toString(s));
		
		timeline.setMaximum(totalSecs);
	}
}
